from sqlalchemy import Column, Integer, Unicode, DateTime,text
from flask_sqlalchemy import SQLAlchemy
from datetime import date, datetime,timezone, timedelta

JST=timezone(timedelta(hours=+9), 'JST')

db = SQLAlchemy()

def setup_db(app):
  '''データベースへの接続設定'''
  dbuser = app.config['DB_USER']
  dbpass = app.config['DB_PASS']
  dbname = app.config['DB_NAME']
  dbhost = app.config['DB_HOST']
  DB_URI=f'mysql+pymysql://{dbuser}:{dbpass}@{dbhost}/{dbname}?charset=utf8'
  app.config["SQLALCHEMY_DATABASE_URI"] = DB_URI
  db.init_app(app)

class Sales:
  def get_sales():
    '''売上データの取得'''
    sql_products = 'select id,name from t_products'
    products = db.engine.execute(sql_products)

    ## 1レコードが [ 製品ID, 日付(2021-01-01 9時), 個数 ] という形式で検索する.
    ## 過去24時間分, 製品IDごとに時間単位での集計を行う.
    sql_product_sales = '''select
      product_id,
      date_format(created_at, "%%Y-%%m-%%d %%H") as sales_date,
      count(*) as count
    from 
      t_sales
    where
      created_at between 
        date_format(date_sub(current_timestamp(), interval 23 HOUR), '%%Y-%%m-%%d %%H:00:00')
      and current_timestamp()
    group by sales_date, product_id
    order by product_id, sales_date'''
    sales = db.engine.execute(sql_product_sales)

    JST=timezone(timedelta(hours=+9), 'JST')
    start_date = datetime.now(tz=JST) - timedelta(hours=23)

    sales_date_list = [(timedelta(hours = h) + start_date).strftime('%Y-%m-%d %H') for h in range(0,24)]
    response = { 
      'sales_date': sales_date_list,
      'products' : { }
    }

    for product in products:
      pid = product['id']
      response['products'][pid] = { 
        'id' : pid,
        'name' : product['name'],
        'sales' : [0 for s in range(len(sales_date_list))]
      }
    ## JSON で結果を返すのに都合の良い形に整形する.
    for data in sales:
      pid = data['product_id']
      sales_date = data['sales_date']
      count = data['count']
      ins_index = sales_date_list.index(sales_date)
      response['products'][pid]['sales'][ins_index] = count

    return response

  def add_sale(product_id):
    '''売上を登録する'''
    current_date = datetime.now(tz=JST).strftime('%Y-%m-%d %H:%M:%S')
    try:
      rs = db.engine.execute(text('select count(*) as count from t_products where id=:id'), id=product_id).fetchone()
      if rs['count'] == 0:
        print("Not exist product_id ({product_id})")
        return False
      
      db.engine.execute(text('insert into t_sales(product_id,created_at) values(:id, :date)'),id=product_id, date=current_date)
    except Exception as e:
      print(e)
      return False

    return True

class Sensors:
  def add_temperature(temperature):
    '''気温データを登録する'''
    current_date = datetime.now(tz=JST).strftime('%Y-%m-%d %H:%M:%S')
    try:
      sql = text('insert into t_temperatures(temperature,created_at) values(:temp,:date)')
      db.engine.execute(sql, temp=temperature,date=current_date)
    except Exception as e:
      print(e)
      return False
    
    return True

  def get_temperatures():
    '''気温データを取得する'''
    result = []
    try:
      ## 気温データは送られてきている全データが検索の対象とする.
      rs = db.engine.execute(text('select temperature, created_at from t_temperatures order by created_at'))
      for row in rs:
        result.append(
          { 'temperature': row['temperature'],
            'date' : row['created_at'].strftime('%Y-%m-%d %H:%M:%S')
          }
        )
    except Exception as e:
      print(e)

    return result
