# app.py
import os
import click
from flask import Flask, render_template
from flask.cli import with_appcontext, AppGroup
from api.model import setup_db
from datetime import date

app = Flask(__name__, 
    instance_relative_config=True,
    static_folder = "./dist/static",
    template_folder = "./dist" )

# 構成ごとに設定ファイルを読み込む.
env = str(app.config['ENV'])
app.config.from_pyfile(f'{env}.cfg')
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False

# データベース用準備.
setup_db(app)

# Blueprint の登録.
from api import sales
from api import sensors
app.register_blueprint( sales.app )
app.register_blueprint( sensors.app )

# '/' にアクセスしたときには所定フォルダのindex.htmlを返す.
# (これはフロントエンド側でビルドされた結果のファイル群)
@app.route('/',methods=['GET'])
def index():
  return render_template('index.html')

if __name__ == "__main__":
  app.run( host="0.0.0.0", port=8080)